#include "opt/suppress_for_vim.hpp"

#include <windows.h>

#include <string>

#include "bind/mode/change_mode.hpp"
#include "core/errlogger.hpp"
#include "core/inputgate.hpp"
#include "core/mode.hpp"
#include "opt/vcmdline.hpp"
#include "util/string.hpp"
#include "util/winwrap.hpp"


namespace vind
{
    namespace opt
    {
        SuppressForVim::SuppressForVim()
        : OptionCreator("suppress_for_vim")
        {}

        void SuppressForVim::do_enable() {
        }

        void SuppressForVim::do_disable() {
        }

        void SuppressForVim::do_process() {
            if(core::get_global_mode() == core::Mode::RESIDENT) {
                return ;
            }

            static HWND pre_hwnd = NULL ;

            auto hwnd = util::get_foreground_window() ;
            if(pre_hwnd == hwnd) {
                return ;
            }

            pre_hwnd = hwnd ;

            auto exename = util::A2a(util::get_module_filename(hwnd)) ;

            if(exename == "win-vind.exe") return ;

            //Whether it is vim
            if(exename.find("vim") != std::string::npos) {
                auto& igate = core::InputGate::get_instance() ;
                igate.close_all_ports() ;
                igate.unabsorb() ;
                core::set_global_mode(core::Mode::RESIDENT) ;
                opt::VCmdLine::print(GeneralMessage("-- RESIDENT --")) ;
            }
            else {
                if(core::get_global_mode() == core::Mode::RESIDENT)
                    bind::ToInsert::sprocess(1, "", true) ;
            }
        }
    }
}
